@@ -20,6 +20,8 @@
 #' @param save_path An optional path in which to save the processed DEM. If left NULL will save it in the same directory as the provided DEM or, if the DEM is a terra object, return only terra objects.
 #' @param n.cores The maximum number of cores to use. Leave NULL to use all cores minus 1.
 #' @param force_update_wbt Whitebox Tools is by default only downloaded if it cannot be found on the computer, and no check are performed to ensure the local version is current. Set to TRUE if you know that there is a new version and you would like to use it.
+#' @param silent_wbt Should Whitebox tools messages be suppressed? This function prints messages to the console already but these messages can be useful if you need to do some debugging.
+
 #'
 #' @return A hydro-processed DEM returned as a terra object and saved to disk if `save_path` is not null.
 #' @export
@@ -43,12 +45,29 @@
 #' terra::plot(res)
 #' }
 
-hydroProcess <- function(DEM, breach_dist, streams = NULL, burn_dist = 10, save_path = NULL, n.cores = NULL, force_update_wbt = FALSE)
+hydroProcess <- function(DEM, breach_dist, streams = NULL, burn_dist = 10, save_path = NULL, n.cores = NULL, force_update_wbt = FALSE, silent_wbt = TRUE)
 {
 
+  #change terra options to allow greater RAM fraction use
+  old <- terra::terraOptions(print = FALSE)
+  terra::terraOptions(memfrac = 0.9)
+  on.exit(terra::terraOptions(memfrac = old$memfrac), add = TRUE)
+
+  if (silent_wbt) {
+    old_option <- whitebox::wbt_verbose()
+    whitebox::wbt_verbose(FALSE)
+    if (!is.null(old_option)) {
+      if (old_option) {
+        on.exit(options("whitebox.verbose_mode" = TRUE))
+      }
+    } else {
+      on.exit(whitebox::wbt_verbose(TRUE), add = TRUE)
+    }
+  }
+
   #initial checks
   rlang::check_installed("whitebox", reason = "required to use function drainageBasins") #This is here because whitebox is not a 'depends' of this package; it is only necessary for this function and is therefore in "suggests"
-  wbtCheck(force = force_update_wbt)  #Check whitebox binaries existence and version, install if necessary or if force_update_wbt = TRUE.
+  invisible(utils::capture.output(wbtCheck(force = force_update_wbt, silent = TRUE)))  #Check whitebox binaries existence and version, install if necessary or if force_update_wbt = TRUE.
 
   # Change whitebox max core options to user request
   cores <- parallel::detectCores()
@@ -80,6 +99,7 @@ hydroProcess <- function(DEM, breach_dist, streams = NULL, burn_dist = 10, save_
   }
 
   #Fill single cell pits
+  message("Filling small pits in the DEM...")
   whitebox::wbt_fill_single_cell_pits(dem = dem_path,
                                       output = paste0(temp_dir, "/filled_single_cells.tif"))
   filled_single_cells <- terra::rast(paste0(temp_dir, "/filled_single_cells.tif"))
